import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:dm/dm.dart';
import 'package:source_gen/source_gen.dart';

class Dependency {
  final String import;
  final String module;
  final String typeName;
  final List<String> parameters;
  final String function;
  final bool isSingleton;
  final bool isFuture;

  Dependency(
    this.module,
    this.typeName,
    this.parameters,
    _isSingleton,
    this.isFuture,
    this.function,
    this.import,
  ) : isSingleton = !_isSingleton && isFuture ? true : _isSingleton {
    if (!_isSingleton && isFuture) {
      print(
          "not singleton future is not supported\n$typeName singleton forcibly");
    }
  }

  static Dependency fromMethod(ClassElement module, MethodElement method) {
    var returnType = method.returnType;
    var isFuture = false;

    if (returnType.isDartAsyncFuture) {
      isFuture = true;
      returnType = (returnType as InterfaceType).typeArguments.first;
    }

    return Dependency(
      module.name,
      returnType.getDisplayString(withNullability: false),
      method.parameters.map((item) {
        return item.type.getDisplayString(withNullability: false);
      }).toList(),
      _singletonChecker.hasAnnotationOf(method),
      isFuture,
      method.name,
      returnType.element.library.librarySource.uri.toString(),
    );
  }

  static const _singletonChecker = TypeChecker.fromRuntime(Singleton);
}
